# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

ud_exit <- function() {
    invisible(.Call('_units_ud_exit', PACKAGE = 'units'))
}

ud_init <- function(path) {
    invisible(.Call('_units_ud_init', PACKAGE = 'units', path))
}

ud_set_encoding <- function(enc_str) {
    invisible(.Call('_units_ud_set_encoding', PACKAGE = 'units', enc_str))
}

ud_compare <- function(x, y, xn, yn) {
    .Call('_units_ud_compare', PACKAGE = 'units', x, y, xn, yn)
}

ud_convert <- function(val, from, to) {
    .Call('_units_ud_convert', PACKAGE = 'units', val, from, to)
}

ud_map_names <- function(names, inunit) {
    invisible(.Call('_units_ud_map_names', PACKAGE = 'units', names, inunit))
}

ud_unmap_names <- function(names) {
    invisible(.Call('_units_ud_unmap_names', PACKAGE = 'units', names))
}

ud_map_symbols <- function(symbols, inunit) {
    invisible(.Call('_units_ud_map_symbols', PACKAGE = 'units', symbols, inunit))
}

ud_unmap_symbols <- function(symbols) {
    invisible(.Call('_units_ud_unmap_symbols', PACKAGE = 'units', symbols))
}

R_ut_get_dimensionless_unit_one <- function() {
    .Call('_units_R_ut_get_dimensionless_unit_one', PACKAGE = 'units')
}

R_ut_new_base_unit <- function() {
    .Call('_units_R_ut_new_base_unit', PACKAGE = 'units')
}

R_ut_new_dimensionless_unit <- function() {
    .Call('_units_R_ut_new_dimensionless_unit', PACKAGE = 'units')
}

R_ut_get_name <- function(unit) {
    .Call('_units_R_ut_get_name', PACKAGE = 'units', unit)
}

R_ut_get_symbol <- function(unit) {
    .Call('_units_R_ut_get_symbol', PACKAGE = 'units', unit)
}

R_ut_are_convertible <- function(a, b) {
    .Call('_units_R_ut_are_convertible', PACKAGE = 'units', a, b)
}

R_ut_scale <- function(unit, factor) {
    .Call('_units_R_ut_scale', PACKAGE = 'units', unit, factor)
}

R_ut_offset <- function(unit, origin) {
    .Call('_units_R_ut_offset', PACKAGE = 'units', unit, origin)
}

R_ut_multiply <- function(a, b) {
    .Call('_units_R_ut_multiply', PACKAGE = 'units', a, b)
}

R_ut_invert <- function(a) {
    .Call('_units_R_ut_invert', PACKAGE = 'units', a)
}

R_ut_divide <- function(numer, denom) {
    .Call('_units_R_ut_divide', PACKAGE = 'units', numer, denom)
}

R_ut_raise <- function(a, i) {
    .Call('_units_R_ut_raise', PACKAGE = 'units', a, i)
}

R_ut_root <- function(a, i) {
    .Call('_units_R_ut_root', PACKAGE = 'units', a, i)
}

R_ut_log <- function(a, base) {
    .Call('_units_R_ut_log', PACKAGE = 'units', a, base)
}

R_ut_parse <- function(name) {
    .Call('_units_R_ut_parse', PACKAGE = 'units', name)
}

R_ut_format <- function(p, names = FALSE, definition = FALSE, ascii = FALSE) {
    .Call('_units_R_ut_format', PACKAGE = 'units', p, names, definition, ascii)
}

