# pillar methods are available for units objects

    Code
      pillar::pillar(x)
    Output
      <pillar>
      [km]
         1
         2
         3
    Code
      pillar::pillar(m)
    Output
      <pillar>
      <mixed_units>
             1 [km]
             2 [km]
             3 [km]
              4 [g]
              5 [g]
              6 [g]

